import tensorflow as tf
physical_devices = tf.config.experimental.list_physical_devices('GPU')
if len(physical_devices) > 0:
    tf.config.experimental.set_memory_growth(physical_devices[0], True)

from tensorflow.python.saved_model import tag_constants
from PIL import Image
import cv2
import numpy as np
import os
import imutils

class Model:

    def __init__(self, base_directory):
        """
        Initialisation method for the model. It can for example be used for loading modules that have to be kept in
        memory or setting up connections.

        :param str base_directory: absolute path to the directory where the model.py file is located.
        """

        self.base_directory = base_directory

    def request(self, data):
        """
        Method for model requests, called separately for each individual request.

        :param dict data: Python dictionary with model input data. In case of models with structured data, dict keys are
            the input fields of the model as defined upon model creation via the platform. In case of a model
            with blob input, the dictionary will contain the following keys:
              - input_blob_path: the absolute location of the input file
              - input_original_filename: the original name of the input blob

        :return dict prediction: Python dictionary with the output fields as defined on model creation. If a model has
        blob output the only key will be 'output_blob'.
        """
        #this is a workaround bc of a bug in Xenia
        os.chdir(self.base_directory)
        import core.utils as utils
        from core.yolov4 import filter_boxes
        #end workaround
        
        self.input_size = 416
        saved_model_loaded = tf.saved_model.load('./checkpoints/yolov4-416', tags=[tag_constants.SERVING])
        infer = saved_model_loaded.signatures['serving_default']
        
        self.model = infer
        
        original_image = cv2.imread(data["image_input"])
        original_image = imutils.resize(original_image, height=1080) #added so that every input image is the same size
        original_image = cv2.cvtColor(original_image, cv2.COLOR_BGR2RGB)

        # image_data = utils.image_preprocess(np.copy(original_image), [input_size, input_size])
        image_data = cv2.resize(original_image, (self.input_size, self.input_size))
        image_data = image_data / 255.
        # image_data = image_data[np.newaxis, ...].astype(np.float32)

        images_data = []
        for i in range(1):
            images_data.append(image_data)
        images_data = np.asarray(images_data).astype(np.float32)


        batch_data = tf.constant(images_data)
        pred_bbox = self.model(batch_data)
        for key, value in pred_bbox.items():
            boxes = value[:, :, 0:4]
            pred_conf = value[:, :, 4:]

        boxes, scores, classes, valid_detections = tf.image.combined_non_max_suppression(
            boxes=tf.reshape(boxes, (tf.shape(boxes)[0], -1, 1, 4)),
            scores=tf.reshape(
                pred_conf, (tf.shape(pred_conf)[0], -1, tf.shape(pred_conf)[-1])),
            max_output_size_per_class=50,
            max_total_size=50,
            iou_threshold=0.45,
            score_threshold=0.25
        )
        pred_bbox = [boxes.numpy(), scores.numpy(), classes.numpy(), valid_detections.numpy()]
        image = utils.draw_bbox(original_image, pred_bbox)
        # image = utils.draw_bbox(image_data*255, pred_bbox)
        image = Image.fromarray(image.astype(np.uint8))
        #image.show()
        image = cv2.cvtColor(np.array(image), cv2.COLOR_BGR2RGB)
        cv2.imwrite('output.jpg', image)
        # Convert to a 4D tensor to feed into the  model


        # Here we set our output parameters in the form of a python dict
        return {'image_output': "output.jpg"}
        

